from typing import Any
import html

from .label_matcher import ActionLabelMatcher
from .nina_api import NinaAPI
from .warning import Warning


class WarningFetcher:
    """Class to fetch warnings for a region."""

    def __init__(self, api: NinaAPI, label_matcher: ActionLabelMatcher):
        """Initialize."""
        self._api = api
        self._label_matcher = label_matcher

    async def fetch_warnings(self, region_code: str) -> list[Warning]:
        """Fetch warnings for the given region."""
        warnings: list[Warning] = []

        raw_warnings = await self._api.get_warnings(region_code)

        for warning_data in raw_warnings:
            warning_details = await self._api.get_warning_details(
                warning_data["payload"]["id"]
            )
            warnings.append(self._build_from_data(warning_data, warning_details))

        return warnings

    def _build_from_data(
        self, warning_data: dict[str, Any], warning_details: dict[str, Any]
    ) -> Warning:
        """Build a Warning object from the data."""
        infos = warning_details["info"][0]

        warning_id: str = warning_data["payload"]["id"]
        headline: str = warning_data["payload"]["data"]["headline"]
        severity: str = warning_data["payload"]["data"]["severity"]

        description: str | None = (
            html.unescape(infos.get("description", ""))
            if "description" in infos
            else None
        )
        sender: str | None = infos.get("senderName", None)
        areas: list[str] = [area["areaDesc"] for area in infos.get("area", [])]
        web: str | None = infos.get("web", None)
        time_send: str = warning_data["sent"]
        time_start: str | None = warning_data.get(
            "effective", warning_data.get("onset", None)
        )
        time_expire: str | None = warning_data.get("expires", None)

        recommended_actions: list[str] = (
            [infos.get("instruction", "").replace("<br/>", " ")]
            if "instruction" in infos
            else []
        )

        if len(recommended_actions) == 0:
            recommended_actions_codes: list[str] = [
                parameter["value"].split(" ")
                for parameter in infos.get("parameter", {})
                if parameter["valueName"] == "instructionCode"
            ]
            recommended_actions = [
                self._label_matcher.match(code) for code in recommended_actions_codes
            ]

        return Warning(
            warning_id,
            headline,
            severity,
            description,
            sender,
            areas,
            recommended_actions,
            web,
            time_send,
            time_start,
            time_expire,
        )



import json
import pathlib
from typing import Any


def load_fixture(filename: str) -> Any:
    """Load a fixtures from files."""

    script_dir = pathlib.Path(__file__).parent.resolve()
    fixture_path = script_dir / "fixtures" / filename

    with open(fixture_path, "r", encoding="utf-8") as file:
        return json.load(file)



from .nina_api import NinaAPI


class ActionLabelMatcher:
    """Class to match action labels with their codes."""

    def __init__(self, api: NinaAPI):
        """Initialize."""
        self._api: NinaAPI = api

        self._labels: dict[str, str] = {}

    @property
    def initiated(self) -> bool:
        """Check if matcher has been initiated."""
        return len(self._labels.items()) > 0

    async def init(self) -> None:
        """Update the labels."""
        self._labels = await self._api.get_labels()

    def match(self, code: str) -> str:
        """Match a code to a label."""
        return self._labels.get(code, code)



from typing import Any

from pynina.api_client import APIClient
from pynina.const import (
    ENDPOINT_DE_LABELS,
    ENDPOINT_NINA_BASE,
    ENDPOINT_REGIONAL_CODE,
    ENDPOINT_WARNING_DETAIL,
    _LOGGER,
)


class NinaAPI:
    """Class to bundle NINA API requests."""

    def __init__(self, client: APIClient):
        """Initialize."""
        self._client = client

    async def get_warnings(self, region_code: str) -> list[dict[str, Any]]:
        """Fetch warnings for the given region."""
        _LOGGER.debug(f"Update region: {region_code}")
        url: str = ENDPOINT_NINA_BASE + region_code + ".json"
        data = await self._client.make_request(url)
        return data

    async def get_warning_details(self, warning_id: str) -> dict[str, str]:
        """Fetch warning details for the given warning."""
        _LOGGER.debug(f"Fetch details for {warning_id}")
        url: str = ENDPOINT_WARNING_DETAIL + warning_id + ".json"
        data = await self._client.make_request(url)
        return data

    async def get_region_codes(self) -> dict[str, Any]:
        """Fetch region codes."""
        _LOGGER.debug("Get all regional codes")
        return await self._client.make_request(ENDPOINT_REGIONAL_CODE)

    async def get_labels(self) -> dict[str, str]:
        """Fetch labels for recommended actions."""
        _LOGGER.debug("Get all action labels")
        return await self._client.make_request(ENDPOINT_DE_LABELS)



from unittest.mock import patch

import pytest

from pynina import Nina, Warning
from tests import load_fixture


@pytest.mark.asyncio
async def test_nina_update_no_regions():
    """Test if the update method handles no regions correctly."""
    with (
        patch("pynina.warning_fetcher.WarningFetcher.fetch_warnings") as fetcher_mock,
        patch("pynina.label_matcher.ActionLabelMatcher.init") as matcher_mock,
    ):
        nina = Nina()

        assert len(nina.warnings) == 0

        await nina.update()

        assert len(nina.warnings) == 0
        fetcher_mock.assert_not_called()
        matcher_mock.assert_awaited_once()


@pytest.mark.asyncio
async def test_nina_update_with_regions():
    """Test if the update method handles multiple regions updates."""
    dummy_warning = Warning(
        "id",
        "headline",
        "severity",
        "description",
        "sender",
        [],
        [],
        "web",
        "sent",
        "start",
        "expires",
    )

    with (
        patch(
            "pynina.warning_fetcher.WarningFetcher.fetch_warnings",
            return_value=[dummy_warning, dummy_warning],
        ) as fetcher_mock,
        patch("pynina.label_matcher.ActionLabelMatcher.init") as matcher_mock,
    ):
        nina = Nina()

        assert len(nina.warnings) == 0

        regions = ["10000", "20000"]

        for region in regions:
            nina.add_region(region)

        await nina.update()

        assert len(nina.warnings) == len(regions)
        matcher_mock.assert_awaited_once()

        for region in regions:
            fetcher_mock.assert_any_call(region)


@pytest.mark.asyncio
async def test_regional_codes():
    """Test if the regional codes are fetched correctly and the codes are shorted properly."""
    region_data = load_fixture("region_codes.json")

    with patch("pynina.nina_api.NinaAPI.get_region_codes", return_value=region_data):
        nina = Nina()

        result = await nina.get_all_regional_codes()
        assert len(result)

        assert result["Waltenhofen (Oberallgäu - Bayern)"] == "097800000000"


@pytest.mark.asyncio
async def test_regional_codes_add_county():
    """Test if counties are added to the list correctly."""
    region_data = load_fixture("region_codes.json")

    with patch("pynina.nina_api.NinaAPI.get_region_codes", return_value=region_data):
        nina = Nina()

        result = await nina.get_all_regional_codes()

        assert (
            result["Brunsbüttel, Stadt (Dithmarschen - Schleswig-Holstein)"]
            == "010510000000"
        )



from copy import deepcopy
from datetime import datetime, timedelta

from pynina import Warning

dummy_warning = Warning(
    "id",
    "headline",
    "severity",
    "description",
    "sender",
    [],
    [],
    "web",
    datetime.now().isoformat(),
    None,
    None,
)


def test_warning_valid_no_start_no_exp():
    """Test if a warning is valid when it has no start and no expiration time."""
    warning = deepcopy(dummy_warning)

    assert warning.is_valid


def test_warning_valid_no_exp():
    """Test if a warning is valid when it has a start but no expiration time."""
    warning = deepcopy(dummy_warning)
    warning.start = (datetime.now() - timedelta(hours=2)).isoformat()

    assert warning.is_valid


def test_warning_valid_no_start_exp_future():
    """Test if a warning is valid when it has an expiration time in the future but start time."""
    warning = deepcopy(dummy_warning)
    warning.expires = (datetime.now() + timedelta(hours=2)).isoformat()

    assert warning.is_valid


def test_warning_valid_no_start_exp_past():
    """Test if a warning is valid when it has an expiration time in the past but start time."""
    warning = deepcopy(dummy_warning)
    warning.expires = (datetime.now() - timedelta(hours=2)).isoformat()

    assert not warning.is_valid


def test_warning_valid_no_start_exp_now():
    """Test if a warning is valid when it has an expiration time is now but start time."""
    warning = deepcopy(dummy_warning)
    warning.expires = datetime.now().isoformat()

    assert not warning.is_valid



from logging import Logger, getLogger, NullHandler

ENDPOINT_REGIONAL_CODE: str = "https://www.xrepository.de/api/xrepository/urn:de:bund:destatis:bevoelkerungsstatistik:schluessel:rs_2021-07-31/download/Regionalschl_ssel_2021-07-31.json"
ENDPOINT_NINA_BASE: str = "https://warnung.bund.de/api31/dashboard/"
ENDPOINT_WARNING_DETAIL: str = "https://warnung.bund.de/api31/warnings/"
ENDPOINT_DE_LABELS: str = (
    "https://warnung.bund.de/api/appdata/gsb/labels/de_labels.json"
)

_LOGGER: Logger = getLogger(__package__)
_LOGGER.addHandler(NullHandler())

CITY_STATES_CODE = ["02", "11"]

COUNTIES = {
    "01051": "Dithmarschen - Schleswig-Holstein",
    "01053": "Herzogtum Lauenburg - Schleswig-Holstein",
    "01054": "Nordfriesland - Schleswig-Holstein",
    "01055": "Ostholstein - Schleswig-Holstein",
    "01056": "Pinneberg - Schleswig-Holstein",
    "01057": "Plön - Schleswig-Holstein",
    "01058": "Rendsburg-Eckernförde - Schleswig-Holstein",
    "01059": "Schleswig-Flensburg - Schleswig-Holstein",
    "01060": "Segeberg - Schleswig-Holstein",
    "01061": "Steinburg - Schleswig-Holstein",
    "01062": "Stormarn - Schleswig-Holstein",
    "03151": "Gifhorn - Niedersachsen",
    "03153": "Goslar - Niedersachsen",
    "03154": "Helmstedt - Niedersachsen",
    "03155": "Northeim - Niedersachsen",
    "03157": "Peine - Niedersachsen",
    "03158": "Wolfenbüttel - Niedersachsen",
    "03159": "Göttingen - Niedersachsen",
    "03241": "Hannover, Region - Niedersachsen",
    "03251": "Diepholz - Niedersachsen",
    "03252": "Hameln-Pyrmont - Niedersachsen",
    "03254": "Hildesheim - Niedersachsen",
    "03255": "Holzminden - Niedersachsen",
    "03256": "Nienburg/Weser - Niedersachsen",
    "03257": "Schaumburg - Niedersachsen",
    "03351": "Celle - Niedersachsen",
    "03352": "Cuxhaven - Niedersachsen",
    "03353": "Harburg - Niedersachsen",
    "03354": "Lüchow-Dannenberg - Niedersachsen",
    "03355": "Lüneburg - Niedersachsen",
    "03356": "Osterholz - Niedersachsen",
    "03357": "Rotenburg (Wümme) - Niedersachsen",
    "03358": "Heidekreis - Niedersachsen",
    "03359": "Stade - Niedersachsen",
    "03360": "Uelzen - Niedersachsen",
    "03361": "Verden - Niedersachsen",
    "03451": "Ammerland - Niedersachsen",
    "03452": "Aurich - Niedersachsen",
    "03453": "Cloppenburg - Niedersachsen",
    "03454": "Emsland - Niedersachsen",
    "03455": "Friesland - Niedersachsen",
    "03456": "Grafschaft Bentheim - Niedersachsen",
    "03457": "Leer - Niedersachsen",
    "03458": "Oldenburg - Niedersachsen",
    "03459": "Osnabrück - Niedersachsen",
    "03460": "Vechta - Niedersachsen",
    "03461": "Wesermarsch - Niedersachsen",
    "03462": "Wittmund - Niedersachsen",
    "05154": "Kleve - Nordrhein-Westfalen",
    "05158": "Mettmann - Nordrhein-Westfalen",
    "05162": "Rhein-Kreis Neuss - Nordrhein-Westfalen",
    "05166": "Viersen - Nordrhein-Westfalen",
    "05170": "Wesel - Nordrhein-Westfalen",
    "05334": "Aachen, Städteregion - Nordrhein-Westfalen",
    "05358": "Düren - Nordrhein-Westfalen",
    "05362": "Rhein-Erft-Kreis - Nordrhein-Westfalen",
    "05366": "Euskirchen - Nordrhein-Westfalen",
    "05370": "Heinsberg - Nordrhein-Westfalen",
    "05374": "Oberbergischer Kreis - Nordrhein-Westfalen",
    "05378": "Rheinisch-Bergischer Kreis - Nordrhein-Westfalen",
    "05382": "Rhein-Sieg-Kreis - Nordrhein-Westfalen",
    "05554": "Borken - Nordrhein-Westfalen",
    "05558": "Coesfeld - Nordrhein-Westfalen",
    "05562": "Recklinghausen - Nordrhein-Westfalen",
    "05566": "Steinfurt - Nordrhein-Westfalen",
    "05570": "Warendorf - Nordrhein-Westfalen",
    "05754": "Gütersloh - Nordrhein-Westfalen",
    "05758": "Herford - Nordrhein-Westfalen",
    "05762": "Höxter - Nordrhein-Westfalen",
    "05766": "Lippe - Nordrhein-Westfalen",
    "05770": "Minden-Lübbecke - Nordrhein-Westfalen",
    "05774": "Paderborn - Nordrhein-Westfalen",
    "05954": "Ennepe-Ruhr-Kreis - Nordrhein-Westfalen",
    "05958": "Hochsauerlandkreis - Nordrhein-Westfalen",
    "05962": "Märkischer Kreis - Nordrhein-Westfalen",
    "05966": "Olpe - Nordrhein-Westfalen",
    "05970": "Siegen-Wittgenstein - Nordrhein-Westfalen",
    "05974": "Soest - Nordrhein-Westfalen",
    "05978": "Unna - Nordrhein-Westfalen",
    "06431": "Bergstraße - Hessen",
    "06432": "Darmstadt-Dieburg - Hessen",
    "06433": "Groß-Gerau - Hessen",
    "06434": "Hochtaunuskreis - Hessen",
    "06435": "Main-Kinzig-Kreis - Hessen",
    "06436": "Main-Taunus-Kreis - Hessen",
    "06437": "Odenwaldkreis - Hessen",
    "06438": "Offenbach - Hessen",
    "06439": "Rheingau-Taunus-Kreis - Hessen",
    "06440": "Wetteraukreis - Hessen",
    "06531": "Gießen - Hessen",
    "06532": "Lahn-Dill-Kreis - Hessen",
    "06533": "Limburg-Weilburg - Hessen",
    "06534": "Marburg-Biedenkopf - Hessen",
    "06535": "Vogelsbergkreis - Hessen",
    "06631": "Fulda - Hessen",
    "06632": "Hersfeld-Rotenburg - Hessen",
    "06633": "Kassel - Hessen",
    "06634": "Schwalm-Eder-Kreis - Hessen",
    "06635": "Waldeck-Frankenberg - Hessen",
    "06636": "Werra-Meißner-Kreis - Hessen",
    "07131": "Ahrweiler - Rheinland-Pfalz",
    "07132": "Altenkirchen (Westerwald) - Rheinland-Pfalz",
    "07133": "Bad Kreuznach - Rheinland-Pfalz",
    "07134": "Birkenfeld - Rheinland-Pfalz",
    "07135": "Cochem-Zell - Rheinland-Pfalz",
    "07137": "Mayen-Koblenz - Rheinland-Pfalz",
    "07138": "Neuwied - Rheinland-Pfalz",
    "07140": "Rhein-Hunsrück-Kreis - Rheinland-Pfalz",
    "07141": "Rhein-Lahn-Kreis - Rheinland-Pfalz",
    "07143": "Westerwaldkreis - Rheinland-Pfalz",
    "07231": "Bernkastel-Wittlich - Rheinland-Pfalz",
    "07232": "Eifelkreis Bitburg-Prüm - Rheinland-Pfalz",
    "07233": "Vulkaneifel - Rheinland-Pfalz",
    "07235": "Trier-Saarburg - Rheinland-Pfalz",
    "07331": "Alzey-Worms - Rheinland-Pfalz",
    "07332": "Bad Dürkheim - Rheinland-Pfalz",
    "07333": "Donnersbergkreis - Rheinland-Pfalz",
    "07334": "Germersheim - Rheinland-Pfalz",
    "07335": "Kaiserslautern - Rheinland-Pfalz",
    "07336": "Kusel - Rheinland-Pfalz",
    "07337": "Südliche Weinstraße - Rheinland-Pfalz",
    "07338": "Rhein-Pfalz-Kreis - Rheinland-Pfalz",
    "07339": "Mainz-Bingen - Rheinland-Pfalz",
    "07340": "Südwestpfalz - Rheinland-Pfalz",
    "08115": "Böblingen - Baden-Württemberg",
    "08116": "Esslingen - Baden-Württemberg",
    "08117": "Göppingen - Baden-Württemberg",
    "08118": "Ludwigsburg - Baden-Württemberg",
    "08119": "Rems-Murr-Kreis - Baden-Württemberg",
    "08125": "Heilbronn - Baden-Württemberg",
    "08126": "Hohenlohekreis - Baden-Württemberg",
    "08127": "Schwäbisch Hall - Baden-Württemberg",
    "08128": "Main-Tauber-Kreis - Baden-Württemberg",
    "08135": "Heidenheim - Baden-Württemberg",
    "08136": "Ostalbkreis - Baden-Württemberg",
    "08215": "Karlsruhe - Baden-Württemberg",
    "08216": "Rastatt - Baden-Württemberg",
    "08225": "Neckar-Odenwald-Kreis - Baden-Württemberg",
    "08226": "Rhein-Neckar-Kreis - Baden-Württemberg",
    "08235": "Calw - Baden-Württemberg",
    "08236": "Enzkreis - Baden-Württemberg",
    "08237": "Freudenstadt - Baden-Württemberg",
    "08315": "Breisgau-Hochschwarzwald - Baden-Württemberg",
    "08316": "Emmendingen - Baden-Württemberg",
    "08317": "Ortenaukreis - Baden-Württemberg",
    "08325": "Rottweil - Baden-Württemberg",
    "08326": "Schwarzwald-Baar-Kreis - Baden-Württemberg",
    "08327": "Tuttlingen - Baden-Württemberg",
    "08335": "Konstanz - Baden-Württemberg",
    "08336": "Lörrach - Baden-Württemberg",
    "08337": "Waldshut - Baden-Württemberg",
    "08415": "Reutlingen - Baden-Württemberg",
    "08416": "Tübingen - Baden-Württemberg",
    "08417": "Zollernalbkreis - Baden-Württemberg",
    "08425": "Alb-Donau-Kreis - Baden-Württemberg",
    "08426": "Biberach - Baden-Württemberg",
    "08435": "Bodenseekreis - Baden-Württemberg",
    "08436": "Ravensburg - Baden-Württemberg",
    "08437": "Sigmaringen - Baden-Württemberg",
    "09171": "Altötting - Bayern",
    "09172": "Berchtesgadener Land - Bayern",
    "09173": "Bad Tölz-Wolfratshausen - Bayern",
    "09174": "Dachau - Bayern",
    "09175": "Ebersberg - Bayern",
    "09176": "Eichstätt - Bayern",
    "09177": "Erding - Bayern",
    "09178": "Freising - Bayern",
    "09179": "Fürstenfeldbruck - Bayern",
    "09180": "Garmisch-Partenkirchen - Bayern",
    "09181": "Landsberg am Lech - Bayern",
    "09182": "Miesbach - Bayern",
    "09183": "Mühldorf am Inn - Bayern",
    "09184": "München - Bayern",
    "09185": "Neuburg-Schrobenhausen - Bayern",
    "09186": "Pfaffenhofen an der Ilm - Bayern",
    "09187": "Rosenheim - Bayern",
    "09188": "Starnberg - Bayern",
    "09189": "Traunstein - Bayern",
    "09190": "Weilheim-Schongau - Bayern",
    "09271": "Deggendorf - Bayern",
    "09272": "Freyung-Grafenau - Bayern",
    "09273": "Kelheim - Bayern",
    "09274": "Landshut - Bayern",
    "09275": "Passau - Bayern",
    "09276": "Regen - Bayern",
    "09277": "Rottal-Inn - Bayern",
    "09278": "Straubing-Bogen - Bayern",
    "09279": "Dingolfing-Landau - Bayern",
    "09371": "Amberg-Sulzbach - Bayern",
    "09372": "Cham - Bayern",
    "09373": "Neumarkt in der Oberpfalz - Bayern",
    "09374": "Neustadt an der Waldnaab - Bayern",
    "09375": "Regensburg - Bayern",
    "09376": "Schwandorf - Bayern",
    "09377": "Tirschenreuth - Bayern",
    "09471": "Bamberg - Bayern",
    "09472": "Bayreuth - Bayern",
    "09473": "Coburg - Bayern",
    "09474": "Forchheim - Bayern",
    "09475": "Hof - Bayern",
    "09476": "Kronach - Bayern",
    "09477": "Kulmbach - Bayern",
    "09478": "Lichtenfels - Bayern",
    "09479": "Wunsiedel im Fichtelgebirge - Bayern",
    "09571": "Ansbach - Bayern",
    "09572": "Erlangen-Höchstadt - Bayern",
    "09573": "Fürth - Bayern",
    "09574": "Nürnberger Land - Bayern",
    "09575": "Neustadt an der Aisch-Bad Windsheim - Bayern",
    "09576": "Roth - Bayern",
    "09577": "Weißenburg-Gunzenhausen - Bayern",
    "09671": "Aschaffenburg - Bayern",
    "09672": "Bad Kissingen - Bayern",
    "09673": "Rhön-Grabfeld - Bayern",
    "09674": "Haßberge - Bayern",
    "09675": "Kitzingen - Bayern",
    "09676": "Miltenberg - Bayern",
    "09677": "Main-Spessart - Bayern",
    "09678": "Schweinfurt - Bayern",
    "09679": "Würzburg - Bayern",
    "09771": "Aichach-Friedberg - Bayern",
    "09772": "Augsburg - Bayern",
    "09773": "Dillingen an der Donau - Bayern",
    "09774": "Günzburg - Bayern",
    "09775": "Neu-Ulm - Bayern",
    "09776": "Lindau (Bodensee) - Bayern",
    "09777": "Ostallgäu - Bayern",
    "09778": "Unterallgäu - Bayern",
    "09779": "Donau-Ries - Bayern",
    "09780": "Oberallgäu - Bayern",
    "10041": "Saarbrücken, Regionalverband - Saarland",
    "10042": "Merzig-Wadern - Saarland",
    "10043": "Neunkirchen - Saarland",
    "10044": "Saarlouis - Saarland",
    "10045": "Saarpfalz-Kreis - Saarland",
    "10046": "St. Wendel - Saarland",
    "12060": "Barnim - Brandenburg",
    "12061": "Dahme-Spreewald - Brandenburg",
    "12062": "Elbe-Elster - Brandenburg",
    "12063": "Havelland - Brandenburg",
    "12064": "Märkisch-Oderland - Brandenburg",
    "12065": "Oberhavel - Brandenburg",
    "12066": "Oberspreewald-Lausitz - Brandenburg",
    "12067": "Oder-Spree - Brandenburg",
    "12068": "Ostprignitz-Ruppin - Brandenburg",
    "12069": "Potsdam-Mittelmark - Brandenburg",
    "12070": "Prignitz - Brandenburg",
    "12071": "Spree-Neiße - Brandenburg",
    "12072": "Teltow-Fläming - Brandenburg",
    "12073": "Uckermark - Brandenburg",
    "13071": "Mecklenburgische Seenplatte - Mecklenburg-Vorpommern",
    "13072": "Rostock - Mecklenburg-Vorpommern",
    "13073": "Vorpommern-Rügen - Mecklenburg-Vorpommern",
    "13074": "Nordwestmecklenburg - Mecklenburg-Vorpommern",
    "13075": "Vorpommern-Greifswald - Mecklenburg-Vorpommern",
    "13076": "Ludwigslust-Parchim - Mecklenburg-Vorpommern",
    "14521": "Erzgebirgskreis - Sachsen",
    "14522": "Mittelsachsen - Sachsen",
    "14523": "Vogtlandkreis - Sachsen",
    "14524": "Zwickau - Sachsen",
    "14625": "Bautzen - Sachsen",
    "14626": "Görlitz - Sachsen",
    "14627": "Meißen - Sachsen",
    "14628": "Sächsische Schweiz-Osterzgebirge - Sachsen",
    "14729": "Leipzig - Sachsen",
    "14730": "Nordsachsen - Sachsen",
    "15081": "Altmarkkreis Salzwedel - Sachsen-Anhalt",
    "15082": "Anhalt-Bitterfeld - Sachsen-Anhalt",
    "15083": "Börde - Sachsen-Anhalt",
    "15084": "Burgenlandkreis - Sachsen-Anhalt",
    "15085": "Harz - Sachsen-Anhalt",
    "15086": "Jerichower Land - Sachsen-Anhalt",
    "15087": "Mansfeld-Südharz - Sachsen-Anhalt",
    "15088": "Saalekreis - Sachsen-Anhalt",
    "15089": "Salzlandkreis - Sachsen-Anhalt",
    "15090": "Stendal - Sachsen-Anhalt",
    "15091": "Wittenberg - Sachsen-Anhalt",
    "16061": "Eichsfeld - Thüringen",
    "16062": "Nordhausen - Thüringen",
    "16063": "Wartburgkreis - Thüringen",
    "16064": "Unstrut-Hainich-Kreis - Thüringen",
    "16065": "Kyffhäuserkreis - Thüringen",
    "16066": "Schmalkalden-Meiningen - Thüringen",
    "16067": "Gotha - Thüringen",
    "16068": "Sömmerda - Thüringen",
    "16069": "Hildburghausen - Thüringen",
    "16070": "Ilm-Kreis - Thüringen",
    "16071": "Weimarer Land - Thüringen",
    "16072": "Sonneberg - Thüringen",
    "16073": "Saalfeld-Rudolstadt - Thüringen",
    "16074": "Saale-Holzland-Kreis - Thüringen",
    "16075": "Saale-Orla-Kreis - Thüringen",
    "16076": "Greiz - Thüringen",
    "16077": "Altenburger Land - Thüringen",
}


class ReadOnlyClass(type):
    def __setattr__(self, name, value):
        raise ValueError(name)



import asyncio
import json

import pytest
from aiohttp import ClientConnectionError, ClientSession
from aioresponses import aioresponses

from pynina.api_client import APIClient, ApiError

SAMPLE_URL = "https://example.com"
SAMPLE_BODY = {"key": "value"}
SAMPLE_BODY_JSON = json.dumps(SAMPLE_BODY)


@pytest.mark.asyncio
async def test_successful_request():
    client = APIClient()

    with aioresponses() as m:
        m.get(SAMPLE_URL, status=200, body=SAMPLE_BODY_JSON)

        result = await client.make_request(SAMPLE_URL)

        assert result == SAMPLE_BODY


@pytest.mark.asyncio
async def test_failed_request():
    client = APIClient()

    with aioresponses() as m:
        m.get(SAMPLE_URL, status=401)

        with pytest.raises(ApiError, match="Invalid response: 401"):
            await client.make_request(SAMPLE_URL)


@pytest.mark.asyncio
async def test_timed_out_request():
    client = APIClient()

    with aioresponses() as m:
        m.get(SAMPLE_URL, exception=asyncio.TimeoutError())

        with pytest.raises(ApiError, match=f"Could not connect to {SAMPLE_URL}"):
            await client.make_request(SAMPLE_URL)


@pytest.mark.asyncio
async def test_connection_error():
    client = APIClient()

    with aioresponses() as m:
        m.get(SAMPLE_URL, exception=ClientConnectionError())

        with pytest.raises(ApiError, match=f"Could not connect to {SAMPLE_URL}"):
            await client.make_request(SAMPLE_URL)


@pytest.mark.asyncio
async def test_external_session_not_closed():
    session = ClientSession()
    client = APIClient(session)

    with aioresponses() as m:
        m.get(SAMPLE_URL, status=200, body=SAMPLE_BODY_JSON)

        await client.make_request(SAMPLE_URL)

        assert not session.closed



from dataclasses import dataclass
from datetime import datetime

@dataclass
class Warning:
    """Dataclass for a warning."""
    id: str
    headline: str
    severity: str
    description: str | None
    sender: str | None
    affected_areas: list[str]
    recommended_actions: list[str]
    web: str | None
    sent: str
    start: str | None
    expires: str | None

    @property
    def is_valid(self) -> bool:
        """Test if warning is valid."""
        if self.expires is not None:
            current_timestamp: float = datetime.now().timestamp()
            expired_timestamp: float = datetime.fromisoformat(self.expires).timestamp()
            return current_timestamp < expired_timestamp
        return True

    def __repr__(self) -> str:
        return (
            f"{self.id} ({self.sent}): [{self.sender}, {self.start} - "
            f"{self.expires} ({self.sent})] {self.headline}, {self.description} - {self.web}"
        )


import asyncio
from typing import Any, Dict

from aiohttp import ClientConnectionError, ClientSession, ClientTimeout

from .const import _LOGGER


class APIClient:
    """Class to perform basic API requests"""

    def __init__(self, session: ClientSession = None):
        """Constructor."""
        self.session = session

    async def make_request(self, url: str):
        """Retrieve data from API."""
        internal_session: bool = False
        if self.session is None:
            internal_session = True
            self.session = ClientSession()

        _LOGGER.debug(f"Try to fetch {url}")

        try:
            async with self.session.get(url, timeout=ClientTimeout(total=9)) as res:
                if res.status != 200:
                    raise ApiError(f"Invalid response: {res.status}")

                json: Dict[str, Any] = await res.json()
                if internal_session:
                    await self.session.close()
                    self.session = None
                return json
        except (ClientConnectionError, asyncio.TimeoutError):
            if internal_session:
                await self.session.close()
                self.session = None
            raise ApiError(f"Could not connect to {url}")


class ApiError(Exception):
    """Raised when API request ended in error."""

    def __init__(self, status: str):
        """Initialize."""
        super().__init__(status)
        self.status = status



from typing import Any, Dict

from aiohttp import ClientSession

from .api_client import APIClient
from .const import (
    CITY_STATES_CODE,
    COUNTIES,
    _LOGGER,
    ReadOnlyClass,
)
from .label_matcher import ActionLabelMatcher
from .nina_api import NinaAPI
from .warning import Warning
from .warning_fetcher import WarningFetcher


class Nina(metaclass=ReadOnlyClass):
    """Main class to interact with the NINA API"""

    def __init__(self, session: ClientSession | None = None):
        """Initialize."""
        self._api_client: APIClient = APIClient(session)
        self._api: NinaAPI = NinaAPI(self._api_client)
        self._label_matcher: ActionLabelMatcher = ActionLabelMatcher(self._api)
        self._warning_fetcher: WarningFetcher = WarningFetcher(
            self._api, self._label_matcher
        )

        self.warnings: dict[str, list[Warning]] = {}
        self.regions: list = []

    def add_region(self, region_code: str):
        """Add a region to monitor."""
        if region_code not in self.regions:
            self.regions.append(region_code)

    async def update(self):
        """Update the warnings."""
        if not self._label_matcher.initiated:
            await self._label_matcher.init()

        self.warnings.clear()

        for region_code in self.regions:
            _LOGGER.debug(f"Update region: {region_code}")
            self.warnings[region_code] = await self._warning_fetcher.fetch_warnings(
                region_code
            )

    async def get_all_regional_codes(self) -> Dict[str, str]:
        """Fetch all regional codes."""
        _LOGGER.debug("Get all regional codes")
        raw_code_data: dict[str, Any] = await self._api.get_region_codes()

        regional_codes: Dict[str, str] = {}
        for data_block in raw_code_data["daten"]:
            region_id: str = data_block[0]
            name: str = data_block[1]

            if region_id[:5] in COUNTIES:
                name = f"{name} ({COUNTIES[region_id[:5]]})"

            if region_id[:2] not in CITY_STATES_CODE:
                region_id = region_id[: len(region_id) - 7] + "0000000"
                regional_codes[name] = region_id

            if (
                region_id[:2] in CITY_STATES_CODE
                and region_id[:2] + "0" * 10 == region_id
            ):
                regional_codes[name] = region_id

        return regional_codes



from unittest.mock import create_autospec

import pytest

from pynina.api_client import APIClient
from pynina.const import (
    ENDPOINT_DE_LABELS,
    ENDPOINT_NINA_BASE,
    ENDPOINT_REGIONAL_CODE,
    ENDPOINT_WARNING_DETAIL,
)
from pynina.nina_api import NinaAPI


@pytest.mark.asyncio
async def test_get_warnings_url():
    """Test if the correct URL is used for fetching warnings."""
    dummy_data = [{"payload": {"id": "some_id"}}]

    mock = create_autospec(APIClient)
    mock.make_request.return_value = dummy_data

    api = NinaAPI(mock)
    result = await api.get_warnings("10044")

    assert result == dummy_data
    mock.make_request.assert_called_with(ENDPOINT_NINA_BASE + "10044.json")


@pytest.mark.asyncio
async def test_get_warning_details_url():
    """Test if the correct URL is used for fetching warnings details."""
    dummy_data = {"payload": {"id": "some_id"}}

    mock = create_autospec(APIClient)
    mock.make_request.return_value = dummy_data

    api = NinaAPI(mock)
    result = await api.get_warning_details("10044")

    assert result == dummy_data
    mock.make_request.assert_called_with(ENDPOINT_WARNING_DETAIL + "10044.json")


@pytest.mark.asyncio
async def test_get_region_codes_url():
    """Test if the correct URL is used for fetching region codes."""
    dummy_data = {"payload": {"id": "some_id"}}

    mock = create_autospec(APIClient)
    mock.make_request.return_value = dummy_data

    api = NinaAPI(mock)
    result = await api.get_region_codes()

    assert result == dummy_data
    mock.make_request.assert_called_with(ENDPOINT_REGIONAL_CODE)


@pytest.mark.asyncio
async def test_get_labels_url():
    """Test if the correct URL is used for fetching labels."""
    dummy_data = {"payload": {"id": "some_id"}}

    mock = create_autospec(APIClient)
    mock.make_request.return_value = dummy_data

    api = NinaAPI(mock)
    result = await api.get_labels()

    assert result == dummy_data
    mock.make_request.assert_called_with(ENDPOINT_DE_LABELS)



from unittest.mock import create_autospec

import pytest

from pynina import Warning
from pynina.label_matcher import ActionLabelMatcher
from pynina.nina_api import NinaAPI
from pynina.warning_fetcher import WarningFetcher
from tests import load_fixture


@pytest.mark.asyncio
async def test_fetch_warnings():
    """Test if the warnings are correctly generated."""
    matcher = create_autospec(ActionLabelMatcher)
    api = create_autospec(NinaAPI)

    api.get_warnings.return_value = load_fixture("warnings.json")
    api.get_warning_details.return_value = load_fixture("warnings-details.json")

    fetcher = WarningFetcher(api, matcher)

    result = await fetcher.fetch_warnings("10044")

    assert len(result) == 1
    assert result[0] == Warning(
        "mow.DE-BW-S-SE018-20211102-18-001",
        "Corona-Verordnung des Landes: Warnstufe durch Landesgesundheitsamt ausgerufen",
        "Minor",
        "Die Zahl der mit dem Corona-Virus infizierten Menschen steigt gegenwärtig stark an. Es wächst daher die Gefahr einer weiteren Verbreitung der Infektion und - je nach Einzelfall - auch von schweren Erkrankungen.",
        None,
        [
            "Bundesland: Freie Hansestadt Bremen, Land Berlin, Land Hessen, Land Nordrhein-Westfalen, Land Brandenburg, Freistaat Bayern, Land Mecklenburg-Vorpommern, Land Rheinland-Pfalz, Freistaat Sachsen, Land Schleswig-Holstein, Freie und Hansestadt Hamburg, Freistaat Thüringen, Land Niedersachsen, Land Saarland, Land Sachsen-Anhalt, Land Baden-Württemberg"
        ],
        ["Waschen sich regelmäßig und gründlich die Hände."],
        None,
        "2021-11-02T20:07:16+01:00",
        None,
        None,
    )

    api.get_warnings.assert_called_with("10044")
    api.get_warning_details.assert_called_with("mow.DE-BW-S-SE018-20211102-18-001")
